;/* execute me to compile with SAS/C (note: CPU option has (nearly) no effect here!)
;SC ARexx-label.c IGNORE 73 DEBUG=SYMBOLFLUSH STRINGMERGE NOSTKCHK LINK
SC ARexx-label.c IGNORE 73 CPU=68020 OPT OPTTIME STRINGMERGE NOSTKCHK
SLink FROM ARexx-label.o TO ARexx-label SC SD STRIPDEBUG NOICONS
Delete ARexx-label.(o|lnk) QUIET
QUIT
*/
/**************************************************************************

ARexx-label.c  --  GoldED4-Scanner fr ARexx-Labels und Procedures

AUTOR(EN):        Thies Wellpott
ERSTELLUNG:       22.05.1997
COPYRIGHT:        (c) 1997 Thies Wellpott
BETRIEBSSYSTEM:   AmigaOS
COMPILER:         SAS/C 6.58

BESCHREIBUNG:

Scanner fr GoldED Version 4.
bergabeparameter:
- ULONG len       Lnge der aktuellen Textzeile
- STRPTR *text    Zeiger auf aktuelle Textzeile
- ULONG *line     Zeiger auf aktuelle Zeilennummer
Rckgabewerte:
- ULONG return    Lnge des gescannten Textes oder 0 fr keinen Text
- STRPTR *text    Zeiger auf Text

Sucht ARexx-Labels und Procedures. Ein Label kann fhrende Leerzeichen
haben, besteht nur aus gltigen ARexx-Symbolzeichen (A-Z, a-z, 0-9, ., !,
$, _, @, #) direkt gefolgt von ":". Folgt darauf evt. durch Leerzeichen
getrennt ein "PROCEDURE", wird dies durch "[PROC]" gekennzeichnet.


FEHLER/EINSCHRNKUNGEN:

- Kommentare werden nicht bercksichtigt und fhren zu Problemen
- die Maximallnge eines gescannten Textes betrgt 60 Zeichen (wird intern
  berprft und abgefangen)


ENTWICKLUNGSGESCHICHTE:

V1.000   22.05.1997   Thies Wellpott
- erste Version

V1.001   15.04.1998   Thies Wellpott
- allgemeine Scanner-Headerdatei verwendet
- 68020 compiliert

**************************************************************************/

#include "twscanner.h"



ULONG __asm golded_scanner(register __d0 ULONG len, register __a0 STRPTR *text,
      register __a1 ULONG *line)
{
   const char *version = "\0$VER: ARexx-label 1.1 " __AMIGADATE__;
   // Stringkonstante (in Code-Hunk) als Puffer mibrauchen
   STRPTR puffer = "123456789012345678901234567890123456789012345678901234567890";
#define PUFFERLAENGE  60
   STRPTR zeichen = *text;

   // fhrende Leerzeichen ignorieren
   SKIPBLANKS(zeichen, len);

   // mind. "l:"
   if (len >= 2)
   {
      unsigned short laenge;        // braucht kein ULONG zu sein

#define EXTRATEXTLAENGE  7          // Zusatzzeichen und -info

      // Labelname kopieren
      *text = puffer;
      laenge = 0;
      while ( len && (laenge < PUFFERLAENGE - EXTRATEXTLAENGE) && ISAREXXSYM(*zeichen) )
      {
         *puffer++ = *zeichen++;
         laenge++;
         len--;
      } // while

      if ( (laenge == 0) || (*zeichen != ':') )
         return 0;

      zeichen++;           // ":" berspringen
      len--;
      SKIPBLANKS(zeichen, len);

      // auf "PROCEDURE" prfen
      if ( (len >= 9) && STRICMP4(zeichen, 'P', 'R', 'O', 'C') &&
            STRICMP4(zeichen, 'E', 'D', 'U', 'R') && (UPPER(*zeichen) == 'E') )
      {  // Infotext anhngen
         *puffer++ = ' ';
         *puffer++ = '[';
         *puffer++ = 'P';
         *puffer++ = 'R';
         *puffer++ = 'O';
         *puffer++ = 'C';
         *puffer = ']';
         return (ULONG)(laenge + 7);
      } // if
      return (ULONG)laenge;
   } // if (len >= ..)

   return 0;
} // golded_scanner()



/**
#include <string.h>

int main(int argc, char *argv[])
{
   char text[128], *erg, laenge;
   
   strcpy(text, "OPTIONS RESULTSq");
   erg = text;
   laenge = golded_scanner(strlen(text)-1, &erg, 0);
   erg[laenge] = 0;

   strcpy(text, "  hallo=':'q");
   erg = text;
   laenge = golded_scanner(strlen(text)-1, &erg, 0);
   erg[laenge] = 0;

   strcpy(text, "fehler:q");
   erg = text;
   laenge = golded_scanner(strlen(text)-1, &erg, 0);
   erg[laenge] = 0;

   strcpy(text, "  summiere:  q");
   erg = text;
   laenge = golded_scanner(strlen(text)-1, &erg, 0);
   erg[laenge] = 0;

   strcpy(text, "  summe:  PROCEDURE EXPOSE a.q");
   erg = text;
   laenge = golded_scanner(strlen(text)-1, &erg, 0);
   erg[laenge] = 0;

   return 0;
} // main()
**/

