;/* execute me to compile with SAS/C (note: CPU option has (nearly) no effect here!)
;SC C-define_typedef.c IGNORE 73 DEBUG=SYMBOLFLUSH STRINGMERGE NOSTKCHK LINK
SC C-define_typedef.c IGNORE 73 CPU=68020 OPT OPTTIME STRINGMERGE NOSTKCHK
SLink FROM C-define_typedef.o TO C-define_typedef SC SD STRIPDEBUG NOICONS
Delete C-define_typedef.(o|lnk) QUIET
QUIT
*/
/**************************************************************************

C-define_typedef.c  --  GoldED4-Scanner fr C/C++ #define und typedef

AUTOR(EN):        Thies Wellpott
ERSTELLUNG:       22.03.1997
COPYRIGHT:        (c) 1997 Thies Wellpott
BETRIEBSSYSTEM:   AmigaOS
COMPILER:         SAS/C 6.58

BESCHREIBUNG:

Scanner fr GoldED Version 4.
bergabeparameter:
- ULONG len       Lnge der aktuellen Textzeile
- STRPTR *text    Zeiger auf aktuelle Textzeile
- ULONG *line     Zeiger auf aktuelle Zeilennummer
Rckgabewerte:
- ULONG return    Lnge des gescannten Textes oder 0 fr keinen Text
- STRPTR *text    Zeiger auf Text

Sucht #define und typedef. Beide Wrter mssen vollstndig klein geschrieben
sein (Def. von ANSI-C). Leerzeichen davor sind erlaubt.

Defines mit Argumenten werden inkl. ihrem erstem angezeigt, weitere Argumente
werden mit "..)" abgekrzt. Die Namen von Defines werden recht locker
behandelt: ein Define-Name endet bei einem Blank, einer schlieenden Klammer
oder einem Komma nach einer ffnenden Klammer; syntaktisch fehlerhafte Define-
Namen werden hier also toleriert.

typedef-Zeilen mssen am Ende ein Semikolon besitzen, da der Name der sonst
mehrzeiligen Definition prinzipbedingt nicht ermittelt werden kann. Im
mehrzeiligen Fall wird als Name die gesamte erste Zeile (ohne "typedef") in
Anfhrungszeichen genommen. Typedefs werden in der Anzeige mit "[type]" am
Ende gekennzeichnet. Komplexe typedefs werden evt. falsch gescannt, siehe
unten.


FEHLER/EINSCHRNKUNGEN:

- Kommentare sind nur am Ende der Zeile erlaubt
- mehrzeilige typedefs prinzipbedingt nur mit Notbehelf mglich
- typedefs mit C-Symbolzeichen nach dem Namen (z. B. Funktionen wie
  "typedef int (*funktion)(char *, char *);", o. .) falsch
- die Maximallnge eines gescannten Textes betrgt 60 Zeichen (wird intern
  berprft und abgefangen)


ENTWICKLUNGSGESCHICHTE:

V1.000   23.03.1997   Thies Wellpott
- erste Version

V1.001   22.05.1997   Thies Wellpott
- mglicher Pufferberlauf bei typedefs abgefangen
- Quelltextdoku verbessert

V1.002   15.04.1998   Thies Wellpott
- allgemeine Scanner-Headerdatei verwendet
- 68020 compiliert

**************************************************************************/

#include "twscanner.h"



ULONG __asm golded_scanner(register __d0 ULONG len, register __a0 STRPTR *text,
      register __a1 ULONG *line)
{
   const char *version = "\0$VER: C-define_typedef 1.2 " __AMIGADATE__;
   // Stringkonstante (in Code-Hunk) als Puffer mibrauchen
   STRPTR puffer = "123456789012345678901234567890123456789012345678901234567890";
#define PUFFERLAENGE  60
   STRPTR zeichen = *text;

   // fhrende Leerzeichen ignorieren
   SKIPBLANKS(zeichen, len);

   // mind. "#define A" oder "typedef A B"
   if (len >= 9)
   {
      unsigned short laenge;        // braucht kein ULONG zu sein

      if (*zeichen == '#')
      {
         // Zeile beginnt mit "#", also auf "#define" kontrollieren
         zeichen++;
         if ( STRCMP4(zeichen, 'd', 'e', 'f', 'i') && STRCMP2(zeichen, 'n', 'e') && ISBLANK(*zeichen) )
         {
            BOOL klammer;

            zeichen++;
            len -= 8;
            SKIPBLANKS(zeichen, len);

#define EXTRATEXTLAENGE  3

            // Name und evt. erstes Argument des Defines extrahieren: alle Zeichen
            // bis Blank oder bis schlieende (runde) Klammer oder bis Komma
            // nach ffnender Klammer; im letzten Fall mit "..)" abschlieen
            *text = puffer;
            laenge = 0;
            klammer = FALSE;
            while ( len && (laenge < PUFFERLAENGE - EXTRATEXTLAENGE) && !ISBLANK(*zeichen) )
            {
               *puffer++ = *zeichen;
               laenge++;
               len--;
               if (*zeichen == '(')
                  klammer = TRUE;         // ffnende Klammer merken
               else if ( klammer && (*zeichen == ',') )
                  len = 0;                // Komma, also erstes Argument zu Ende
               else if (*zeichen == ')')
               {
                  klammer = FALSE;        // schlieende Klammern merken
                  len = 0;
               } // else if
               zeichen++;
            } // while

            // falls kein Name vorhanden, nicht bernehmen
            if (laenge == 0)
               return 0;

            // Klammer nicht geschlossen, d. h. Define mit mehreren
            // Argumenten, also "..)" anhngen
            if (klammer)
            {
               *puffer++ = '.';
               *puffer++ = '.';
               *puffer = ')';
               laenge += 3;
            } // if

            return laenge;
         } // if ("#define")
      } else
      {
         // Zeile auf "typedef" prfen
         if ( STRCMP4(zeichen, 't', 'y', 'p', 'e') && STRCMP3(zeichen, 'd', 'e', 'f') && ISBLANK(*zeichen) )
         {
            STRPTR ende;

            zeichen++;
            len -= 8;
            SKIPBLANKS(zeichen, len);

            if (len == 0)
               return 0;

            // Semikolon in Zeile suchen (vorwrts, damit Kommentar am Ende
            // der Zeile mglich)
            ende = zeichen;
            while ( (ende - zeichen < len - 1) && (*ende != ';') )
               ende++;

#undef EXTRATEXTLAENGE
#define EXTRATEXTLAENGE  7       // Zusatzinfo

            // Semikolon am Zeilenende?
            if (*ende != ';')
            {
               // kein Semikolon am Ende, also mehrzeilige typedef; kann nicht richtig
               // gescannt werden, gesamte Zeile in Anfhrungszeichen nehmen

               // Leerzeichen am Ende entfernen
               ende = &zeichen[len - 1];
               while ( len && ISBLANK(*ende) )
               {
                  ende--;
                  len--;
               } // while

               *text = puffer;
               if (len > PUFFERLAENGE - EXTRATEXTLAENGE - 2)   // Text auf Pufferlnge beschrnken
                  len = PUFFERLAENGE - EXTRATEXTLAENGE - 2;
               laenge = len + 2;
               *puffer++ = '"';
               while (len)
               {
                  *puffer++ = *zeichen++;
                  len--;
               } // while;
               *puffer++ = '"';
            } else
            {
               // einzeilige typedef, von hinten "ersten" Namen lesen
               STRPTR puffer2;
               short klammer;

               // Semikolon berspringen
               len--;
               ende--;

               // hinten alle nicht C-Symbolzeichen und Ausdrcke in eckigen
               // Klammern berspringen
               klammer = 0;
               while ( len && ((klammer > 0) || !ISCSYM(*ende)) )
               {
                  if (*ende == ']')
                     klammer++;
                  else if (*ende == '[')
                     klammer--;
                  ende--;
                  len--;
               } // while

               // alle C-Symbolzeichen von hinten in Puffer kopieren (rckwrts!)
               puffer += PUFFERLAENGE - EXTRATEXTLAENGE;
               puffer2 = puffer;
               laenge = 0;
               while ( len && (laenge < PUFFERLAENGE - EXTRATEXTLAENGE) && ISCSYM(*ende) )
               {
                  // erst puffer2 ein Zeichen zurck; diese Version erspart
                  // unntiges "- 1" und "+ 1" oben und unten
                  *--puffer2 = *ende--;
                  len--;
                  laenge++;
               } // while
               *text = puffer2;
            } // else

            // Infotext anhngen
            *puffer++ = ' ';
            *puffer++ = '[';
            *puffer++ = 't';
            *puffer++ = 'y';
            *puffer++ = 'p';
            *puffer++ = 'e';
            *puffer = ']';
            return (ULONG)(laenge + 7);
         } // if ("typedef")
      } // else
   } // if (len >= 9)

   return 0;
} // golded_scanner()


/**
#include <string.h>

int main(int argc, char *argv[])
{
   char text[128], *erg, laenge;
   
   strcpy(text, "typedef\tuns short\tUWORD; \t\t /* kommentar * q");
   erg = text;
   laenge = golded_scanner(strlen(text)-1, &erg, 0);
   erg[laenge] = 0;

   strcpy(text, "typedef char *STRPTR;q");
   erg = text;
   laenge = golded_scanner(strlen(text)-1, &erg, 0);
   erg[laenge] = 0;

   strcpy(text, "typedef struct     q");
   erg = text;
   laenge = golded_scanner(strlen(text)-1, &erg, 0);
   erg[laenge] = 0;

   strcpy(text, "typedef int (*funcptr)();q");
   erg = text;
   laenge = golded_scanner(strlen(text)-1, &erg, 0);
   erg[laenge] = 0;

   strcpy(text, "typedef char *table[20];q");
   erg = text;
   laenge = golded_scanner(strlen(text)-1, &erg, 0);
   erg[laenge] = 0;

   return 0;
} // main()
**/

