;/* execute me to compile with SAS/C
;SC SQL-CREATE.c IGNORE 73 DEBUG=SYMBOLFLUSH STRINGMERGE NOSTKCHK LINK
SC SQL-CREATE.c IGNORE 73 CPU=68020 OPT OPTTIME STRINGMERGE NOSTKCHK
SLink FROM SQL-CREATE.o TO SQL-CREATE SC SD STRIPDEBUG NOICONS
Delete SQL-CREATE.(o|lnk) QUIET
QUIT
*/
/**************************************************************************

SQL-CREATE.c  --  GoldED4-Scanner fr SQL CREATE-Anweisungen

AUTOR(EN):        Thies Wellpott
ERSTELLUNG:       15.04.1998
COPYRIGHT:        (c) 1998 Thies Wellpott
BETRIEBSSYSTEM:   AmigaOS
COMPILER:         SAS/C 6.58

BESCHREIBUNG:

Scanner fr GoldED Version 4.
bergabeparameter:
- ULONG len       Lnge der aktuellen Textzeile
- STRPTR *text    Zeiger auf aktuelle Textzeile
- ULONG *line     Zeiger auf aktuelle Zeilennummer
Rckgabewerte:
- ULONG return    Lnge des gescannten Textes oder 0 fr keinen Text
- STRPTR *text    Zeiger auf Text

Sucht SQL-CREATE-Anweisungen. Leerzeichen am Zeilenanfang werden ignoriert,
ein "OR REPLACE" (fr Trigger, Prozeduren und Packages), "UNIQUE" (fr
INDEX), "PUBLIC" (fr SYNONYM, ROLLBACK SEGMENT, u. a.), "FORCE" oder
"NOFORCE" (fr VIEW) nach dem "CREATE" ebenfalls. Die Gro-/Kleinschreibung
ist egal. Das Wort nach "CREATE [OR REPLACE] [UNIQUE|PUBLIC|[NO]FORCE]"
wird als Objekttyp angesehen, von dem die ersten vier Zeichen als erste
Spalte mitausgegeben werden.

Normalerweise werden nur einwortige Objekttypen akzeptiert, behandelte
Ausnahmen sind "PACKAGE [BODY]", "ROLLBACK SEGM[ENT]", "SNAPSHOT [LOG ON]",
"DATABASE LINK" und "SCHEMA AUTH[ORIZATION]". Der Objekttypkrzel wird
jeweils entsprechend angepat (z. B. wird aus "PACK" fr eine Package
"PABO" fr Package Body).

Das dann folgende Wort ist der Objektname. Alle aufeinanderfolgenden
SQL-Symbol-Zeichen (A-Z, a-z, 0-9, _, $, #, .) werden als Name angesehen.



FEHLER/EINSCHRNKUNGEN:

- Kommentare werden nicht bercksichtigt
- die Maximallnge eines gescannten Textes betrgt 60 Zeichen (wird intern
  berprft und abgefangen)


ENTWICKLUNGSGESCHICHTE:

V1.000   15.04.1998   Thies Wellpott
- erste Version

V1.001   16.04.1998   Thies Wellpott
- spezielle Prfixe (PUBLIC, UNIQUE, etc.) und mehrwortige Objekttypen
  werden jetzt abgefangen

**************************************************************************/

#include "twscanner.h"



ULONG __asm golded_scanner(register __d0 ULONG len, register __a0 STRPTR *text,
      register __a1 ULONG *line)
{
   const char *version = "\0$VER: SQL-CREATE 1.1 " __AMIGADATE__;
   // Stringkonstante (in Code-Hunk) als Puffer mibrauchen
   STRPTR puffer = "123456789012345678901234567890123456789012345678901234567890";
#define PUFFERLAENGE  60
   STRPTR zeichen = *text;

   // fhrende Leerzeichen ignorieren
   SKIPBLANKS(zeichen, len);

   // Minimaltext ist: "CREATE T x" (T fr Typ, x fr Bezeichner);
   // Leerzeichen nach "CREATE" nicht testen, falls es vergessen wurde
   if ( (len >= 10) && (STRICMP4(zeichen, 'C', 'R', 'E', 'A') && STRICMP2(zeichen, 'T', 'E')) )
   {
      UWORD laenge;                 // UWORD reicht fr Lnge, ULONG braucht nicht

      len -= 6;                     // "CREATE" ist schon bersprungen worden
      SKIPBLANKS(zeichen, len);

      // evt. "OR REPLACE" berspringen; prfe, ob "OR " vorhanden;
      // Minimaltext "OR REPLACE T x"
      if ( (len >= 14) && IPSTRICMP2(zeichen, 'O', 'R') && ISBLANK(zeichen[2]) )
      {
         zeichen += 3;
         len -= 3;
         SKIPBLANKS(zeichen, len);
         // das nchste Wort (mte "REPLACE" sein) berspringen
         while ( len && ISALPHA(*zeichen) )
         {
            zeichen++;
            len--;
         } // while
      } // if

      // evt. "PUBLIC " berspringen; Minimaltext "PUBLIC T x"
      if ( (len >= 10) && IPSTRICMP4(zeichen, 'P', 'U', 'B', 'L') &&
           IPSTRICMP2(zeichen + 4, 'I', 'C') && ISBLANK(zeichen[6]) )
      {
         zeichen += 7;
         len -= 7;
      } // if
      // oder evt. "UNIQUE " berspringen; Minimaltext "UNIQUE T x"
      else if ( (len >= 10) && IPSTRICMP4(zeichen, 'U', 'N', 'I', 'Q') &&
                IPSTRICMP2(zeichen + 4, 'U', 'E') && ISBLANK(zeichen[6]) )
      {
         zeichen += 7;
         len -= 7;
      } // else if
      // oder evt. "NOFORCE " berspringen; Minimaltext "NOFORCE T x"
      else if ( (len >= 11) && IPSTRICMP4(zeichen, 'N', 'O', 'F', 'O') &&
                IPSTRICMP3(zeichen + 4, 'R', 'C', 'E') && ISBLANK(zeichen[7]) )
      {
         zeichen += 8;
         len -= 8;
      } // else if
      // oder evt. "FORCE " berspringen; Minimaltext "FORCE T x"
      else if ( (len >= 10) && IPSTRICMP4(zeichen, 'F', 'O', 'R', 'C') &&
                (UPPER(zeichen[4]) == 'E') && ISBLANK(zeichen[5]) )
      {
         zeichen += 6;
         len -= 6;
      } // else if

      SKIPBLANKS(zeichen, len);
      // "zeichen" steht jetzt auf Beginn des Objekttyps

      *text = puffer;
      laenge = 0;

      // max. erste vier Zeichen des Objekttyps kopieren
      while ( len && (laenge < 4) && ISALPHA(*zeichen) )
      {
         *puffer++ = UPPER(*zeichen++);
         laenge++;
         len--;
      } // while

      if (laenge == 0)        // wohl kein Typ angegeben
         return 0;

      // restliche Zeichen des Objekttyps berspringen
      while ( len && ISALPHA(*zeichen) )
      {
         zeichen++;
         len--;
      } // while
      SKIPBLANKS(zeichen, len);
      // "zeichen" steht jetzt auf zweitem Objekttypwort oder Objektname

      // Sonderflle von mehrwortigen Typen abfangen; Minimaltext "WORT x" (bei
      // allen mglichen folgenden Wrter sind mind. 4 Buchstaben vorhanden)
      if ( (laenge == 4) && (len >= 6) )
      {
         BOOL skip_alpha = FALSE;   // mssen am Ende nach Buchstaben bersprungen werden?

         // prfe "PACKAGE BODY", mache aus Krzel "PACK" "PABO"
         if ( IPSTRICMP4(puffer - 4, 'P', 'A', 'C', 'K') &&
              IPSTRICMP4(zeichen, 'B', 'O', 'D', 'Y') && ISBLANK(zeichen[4]) )
         {
            puffer[-2] = 'B';
            puffer[-1] = 'O';
            zeichen += 5;
            len -= 5;
         } // if
         // prfe "ROLLBACK SEGMENT", mache aus Krzel "ROLL" "ROBS"
         else if ( IPSTRICMP4(puffer - 4, 'R', 'O', 'L', 'L') &&
                   IPSTRICMP4(zeichen, 'S', 'E', 'G', 'M') )
         {
            puffer[-2] = 'B';
            puffer[-1] = 'S';
            zeichen += 4;
            len -= 4;
            skip_alpha = TRUE;
         } // else if
         // prfe "SNAPSHOT LOG ON", mache aus Krzel "SNAP" "SNLO"
         else if ( IPSTRICMP4(puffer - 4, 'S', 'N', 'A', 'P') &&
                   IPSTRICMP3(zeichen, 'L', 'O', 'G') && ISBLANK(zeichen[3]) )
         {
            puffer[-2] = 'L';
            puffer[-1] = 'O';
            zeichen += 4;
            len -= 4;
            SKIPBLANKS(zeichen, len);     // Leerzeichen bis "ON" berspringen
            skip_alpha = TRUE;
         } // else if
         // prfe "DATABASE LINK", mache aus Krzel "DATA" "DABL"
         else if ( IPSTRICMP4(puffer - 4, 'D', 'A', 'T', 'A') &&
                   IPSTRICMP4(zeichen, 'L', 'I', 'N', 'K') && ISBLANK(zeichen[4]) )
         {
            puffer[-2] = 'B';
            puffer[-1] = 'L';
            zeichen += 5;
            len -= 5;
         } // if
         // prfe "SCHEMA AUTOHORIZATION", mache aus Krzel "SCHE" "SCAU"
         else if ( IPSTRICMP4(puffer - 4, 'S', 'C', 'H', 'E') &&
                   IPSTRICMP4(zeichen, 'A', 'U', 'T', 'H') )
         {
            puffer[-2] = 'A';
            puffer[-1] = 'U';
            zeichen += 4;
            len -= 4;
            skip_alpha = TRUE;
         } // else if

         // wenn erforderlich, restliche Zeichen des Objekttyps berspringen
         if (skip_alpha)
         {
            while ( len && ISALPHA(*zeichen) )
            {
               zeichen++;
               len--;
            } // while
         } // if
         SKIPBLANKS(zeichen, len);
      } // if

      // "zeichen" steht jetzt auf Objektname

      // Objekttypkrzel mit Leerzeichen auffllen bis insgesamt 5 Zeichen
      // geschrieben wurden
      do
      {
         *puffer++ = ' ';
         laenge++;
      } while (laenge < 5);

      if (!ISSQLSYM(*zeichen))      // Name beginnt nicht mit gltigem Zeichen?
         return 0;

      // Objektname kopieren
      while ( len && (laenge < PUFFERLAENGE) && ISSQLSYM(*zeichen) )
      {
         *puffer++ = *zeichen++;
         laenge++;
         len--;
      } // while

      return (ULONG)laenge;
   } // if

   return 0;
} // golded_scanner()



/**
#include <string.h>

int main(int argc, char *argv[])
{
   char text[128], *erg, laenge;
   
   strcpy(text, "crEATe table hummel");
   erg = text;
   laenge = golded_scanner(strlen(text), &erg, 0);
   erg[laenge] = 0;

   strcpy(text, "CREATE SEQUENCE Seq_Bereich MAXVALUE 999999 CYCLE;");
   erg = text;
   laenge = golded_scanner(strlen(text), &erg, 0);
   erg[laenge] = 0;

   strcpy(text, "  CREATE	 TABLE		Benutzergruppe_enthaltene_Bgrp$#%&");
   erg = text;
   laenge = golded_scanner(strlen(text), &erg, 0);
   erg[laenge] = 0;

   strcpy(text, "	CREATE  uniQUe  INDEX Bgrp_enth_Bgrp_Benutzergruppe ON");
   erg = text;
   laenge = golded_scanner(strlen(text), &erg, 0);
   erg[laenge] = 0;

   strcpy(text, "CreATE  snapshot  log   on   asb_Benutzer;");
   erg = text;
   laenge = golded_scanner(strlen(text), &erg, 0);
   erg[laenge] = 0;

   strcpy(text, "CREATE USER asb_ODBC_Abwesenheit IDENTIFIED BY asb_ODBC_Abwesenheit");
   erg = text;
   laenge = golded_scanner(strlen(text), &erg, 0);
   erg[laenge] = 0;

   strcpy(text, "CREATE OR REPLACE PROCEDURE asb.Exec_DDL(string VARCHAR2) AS");
   erg = text;
   laenge = golded_scanner(strlen(text), &erg, 0);
   erg[laenge] = 0;

   return 0;
} // main()
**/

